package com.cim.idm.framework.exception;

import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;

import org.apache.commons.lang3.StringUtils;
import org.springframework.core.NestedRuntimeException;
import org.springframework.dao.DataAccessException;

/*
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.exception
 *  NAME    : ErrorSignal.java
 *  TYPE    : JAVA
 *  DESCRIPTION :
 *
 ****************************************************************************
 */

public class ErrorSignal
{
	public final static String	NotActive						= "NotActive";
	public final static String	CouldNotMatchData				= "CouldNotMatchData:DataInfo";
	public final static String	InvalidQueryType				= "InvalidQueryType";
	public final static String	InvalidQueryState				= "InvalidQueryState";
	public final static String	NullPointKeySignal				= "NullPointKeySignal";
	public final static String	UnexprectedSignal				= "UnexprectedSignal";
	public final static String	InvalidArgumentSignal			= "InvalidArgumentSignal";
	public final static String	NoDefineServiceBean				= "NoDefineServiceBean";
	public final static String	NoServiceRegistered				= "NoServiceRegistered";
	public final static String	NoServiceBeanRegistered			= "NoServiceBeanRegistered";
	public final static String	NoBeanDefinition				= "NoBeanDefinition";
	public final static String	NotDefineObjectAttributeSignal	= "NotDefineObjectAttributeSignal";

	public static final String	DataAccessException				= "DataAccessException";
	public final static String	DuplicateNameSignal				= "DuplicateNameSignal";
	public final static String	NotFoundSignal					= "NotFoundSignal";

	public final static String	InvalidMessageType				= "InvalidMessageType";
	
	public final static String	InvalidParameter				= "InvalidParameter";

	public static IDMFrameErrorSignal getNotifyException(Throwable e)
	{
		if (e instanceof IDMFrameErrorSignal)
			return (IDMFrameErrorSignal) e;
		else if (e instanceof InvocationTargetException)
		{
			Throwable cause = ((InvocationTargetException) e).getTargetException();
			return getNotifyException(cause);
		}
		else if (e instanceof SQLException)
		{
			return getNotifyException((SQLException) e, null, null);
		}
		else if (e instanceof DataAccessException)
		{
			return getNotifyException((DataAccessException) e);
		}
		else if (e instanceof NestedRuntimeException)
		{
			Throwable cause = ((NestedRuntimeException) e).getMostSpecificCause();
			if (cause != null)
				return new IDMFrameErrorSignal(ErrorSignal.UnexprectedSignal, e.getMessage(), e);
			else
			{
				cause = e.getCause();
				if (cause != null)
					return new IDMFrameErrorSignal(ErrorSignal.UnexprectedSignal, e.getMessage(), e);
			}
		}

		return new IDMFrameErrorSignal(ErrorSignal.UnexprectedSignal, e.getMessage(), e);
	}

	public static void notifyException(Throwable e)
	{
		throw getNotifyException(e);
	}

	public static IDMFrameDBErrorSignal getNotifyException(DataAccessException e)
	{
		Throwable cause = e.getMostSpecificCause();
		//		Throwable cause = e.getCause();
		if (cause != null && cause instanceof SQLException)
		{
			return getNotifyException((SQLException) cause, null, null);
		}
		return new IDMFrameDBErrorSignal(ErrorSignal.DataAccessException, e);
	}

	public static IDMFrameDBErrorSignal getNotifyException(DataAccessException e, String sql)
	{
		Throwable cause = e.getMostSpecificCause();
		//		Throwable cause = e.getCause();
		if (cause != null && cause instanceof SQLException)
		{
			return getNotifyException((SQLException) cause, null, sql);
		}
		return new IDMFrameDBErrorSignal(ErrorSignal.DataAccessException, sql, e);
	}

	public static IDMFrameDBErrorSignal getNotifyException(DataAccessException e, String bindSet, String sql)
	{
		Throwable cause = e.getMostSpecificCause();
		//		Throwable cause = e.getCause();
		if (cause != null && cause instanceof SQLException)
		{
			return getNotifyException((SQLException) cause, bindSet, sql);
		}
		return new IDMFrameDBErrorSignal(ErrorSignal.DataAccessException, bindSet, sql, e);
	}

	public static IDMFrameDBErrorSignal getNotifyException(SQLException e, String bindSet, String sql)
	{
		String errorCode;
		if (e.getErrorCode() == 1)
			errorCode = ErrorSignal.DuplicateNameSignal;
		else
			errorCode = ErrorSignal.DataAccessException;

		if (StringUtils.isNotEmpty(bindSet) && StringUtils.isNotEmpty(sql))
			return new IDMFrameDBErrorSignal(errorCode, bindSet, sql, e);
		else if (StringUtils.isNotEmpty(sql))
			return new IDMFrameDBErrorSignal(errorCode, sql, e);
		else
			return new IDMFrameDBErrorSignal(errorCode, e);
	}

}
