package com.cim.idm.framework.exception.ext;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

@SuppressWarnings("serial")
public class LocalizedException extends RuntimeException
{
	protected Log		log	= LogFactory.getLog(this.getClass());

	protected String	errorCode;
	protected Object[]	errorArguments;

	public LocalizedException(String errorCode)
	{
		super(ExceptionMessages.getInstance().format(errorCode, null));
		this.errorCode = errorCode;
		this.errorArguments = ExceptionMessages.getInstance().preAppendOperation(new Object[0]);

		writeLog(LogLevel.Info, makeErrorLogMessage(), null);
	}

	public LocalizedException(String errorCode, Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, args));
		this.errorCode = errorCode;
		this.errorArguments = ExceptionMessages.getInstance().preAppendOperation(args);

		writeLog(LogLevel.Info, makeErrorLogMessage(), null);
	}

	public LocalizedException(LogLevel logLevel, String errorCode, Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, args));
		this.errorCode = errorCode;
		this.errorArguments = ExceptionMessages.getInstance().preAppendOperation(args);

		writeLog(logLevel, makeErrorLogMessage(), null);
	}

	public LocalizedException(String errorCode, Throwable throwable)
	{
		//		super(GenericServiceProxy.getExceptionMessages().format(errorCode, null), throwable);
		super(ExceptionMessages.getInstance().format(errorCode, new Object[] { throwable.getMessage() }),
				throwable);
		this.errorCode = errorCode;
		this.errorArguments = ExceptionMessages.getInstance().preAppendOperation(new Object[0]);

		writeLog(LogLevel.Info, makeErrorLogMessage(), throwable);
	}
	public LocalizedException(String errorCode, String errorMessage, Throwable throwable){
		super(errorMessage);
		this.errorCode = errorCode;
		this.errorArguments = new Object[0];
		writeLog(LogLevel.Info,errorMessage,throwable);
	}
	public LocalizedException(String errorCode, Throwable throwable, Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, args), throwable);
		this.errorCode = errorCode;
		this.errorArguments = ExceptionMessages.getInstance().preAppendOperation(args);

		writeLog(LogLevel.Info, makeErrorLogMessage(), throwable);
	}

	@SuppressWarnings("unchecked")
	public LocalizedException(String errorCode, Class serviceClass, String operation, Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, serviceClass, operation, args));
		this.errorCode = errorCode;
		this.errorArguments =
				ExceptionMessages.getInstance().preAppendOperation(serviceClass, operation, args);

		writeLog(LogLevel.Info, makeErrorLogMessage(), null);
	}

	@SuppressWarnings("unchecked")
	public LocalizedException(String errorCode, Throwable throwable, Class serviceClass, String operation,
			Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, serviceClass, operation, args), throwable);
		this.errorCode = errorCode;
		this.errorArguments =
				ExceptionMessages.getInstance().preAppendOperation(serviceClass, operation, args);

		writeLog(LogLevel.Info, makeErrorLogMessage(), throwable);
	}

	@SuppressWarnings("unchecked")
	public LocalizedException(LogLevel logLevel, String errorCode, Throwable throwable, Class serviceClass, String operation,
			Object... args)
	{
		super(ExceptionMessages.getInstance().format(errorCode, serviceClass, operation, args), throwable);
		this.errorCode = errorCode;
		this.errorArguments =
				ExceptionMessages.getInstance().preAppendOperation(serviceClass, operation, args);

		writeLog(logLevel, makeErrorLogMessage(), throwable);
	}
	
	public String getErrorCode()
	{
		return this.errorCode;
	}

	public Object[] getErrorArguments()
	{
		return errorArguments;
	}

	private String makeErrorLogMessage()
	{
		return String.format("%s: %s", this.errorCode, this.getMessage());
	}

	private void writeLog(LogLevel logLevel, Object logMsg, Throwable e)
	{
		switch (logLevel)
		{
			case Fatal:
//				if (log.isDebugEnabled())
					log.fatal(logMsg, e);
//				else
//					log.fatal(logMsg);
				break;
			case Error:
//				if (log.isDebugEnabled())
					log.error(logMsg, e);
//				else
//					log.error(logMsg);
				break;
			case Warn:
				if (log.isDebugEnabled())
					log.warn(logMsg, e);
				else
					log.warn(logMsg);
				break;
			case Info:
				if (log.isDebugEnabled())
					log.info(logMsg, e);
				else
					log.info(logMsg);
				break;
			case Debug:
				if (log.isDebugEnabled())
					log.debug(logMsg, e);
				break;
			case Trace:
				if (log.isTraceEnabled())
					log.trace(logMsg, e);
				break;
		}
	}
}
